# SynchronizedCells.py
# Synchronized circular buffer of integer values

import threading

class SynchronizedCells:

	def __init__( self ):
        	"""Set cells, flags, locations and condition variable"""

              	self.sharedCells = [ -1, -1, -1, -1, -1 ] # buffer
              	self.writeable = 1                 # buffer may be changed
              	self.readable = 0                  # buffer may not be read
              	self.writeLocation = 0             # current writing index
              	self.readLocation = 0              # current reading index

              	self.threadCondition = threading.Condition()

	def setSharedNumber( self, newNumber ):
             	"""Set next buffer index value--blocks until lock acquired"""

              	# block until lock released then acquire lock
		self.threadCondition.acquire()

              	# while buffer is full, release lock and block
              	while not self.writeable:
                	print "WAITING TO PRODUCE", newNumber
              		self.threadCondition.wait()

              	# buffer is not full, lock has been re-acquired

              	# produce a number in shared cells, consumer may consume
              	self.sharedCells[ self.writeLocation ] = newNumber
              	self.readable = 1
              	print "Produced %2d into cell %d" % \
                  	( newNumber, self.writeLocation ),

              	# set writing index to next place in buffer
              	self.writeLocation = ( self.writeLocation + 1 ) % 5

              	print "      write %d read %d " % \
                  	( self.writeLocation, self.readLocation ),
              	print self.sharedCells

              	# if producer has caught up to consumer, buffer is full
              	if self.writeLocation == self.readLocation:
                 	self.writeable = 0
                  	print "BUFFER FULL"

              	self.threadCondition.notify() # wake up a waiting thread
              	self.threadCondition.release() # allow lock to be acquired

	def getSharedNumber( self ):
             	"""Get next buffer index value--blocks until lock acquired"""

              	# block until lock released then acquire lock
              	self.threadCondition.acquire()

              	# while buffer is empty, release lock and block
              	while not self.readable:
                 	print "WAITING TO CONSUME"
              		self.threadCondition.wait()

              	# buffer is not empty, lock has been re-acquired

              	# consume a number from shared cells, producer may produce
              	self.writeable = 1
              	tempNumber = self.sharedCells[ self.readLocation ]
              	self.sharedCells[ self.readLocation ] = -1

              	print "Consumed %2d from cell %d" % \
                 	( tempNumber, self.readLocation ),

              	# move to next produced number
              	self.readLocation = ( self.readLocation + 1 ) % 5

              	print "   write %d read %d " % \
                 	( self.writeLocation, self.readLocation ),
              	print self.sharedCells

              	# if consumer has caught up to producer, buffer is empty
              	if self.readLocation == self.writeLocation:
                 	self.readable = 0
                 	print "BUFFER EMPTY"

              	self.threadCondition.notify() # wake up a waiting thread
              	self.threadCondition.release() # allow lock to be acquired

              	return tempNumber
